package com.vendor.burn.views;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.vendor.burn.Messages;
import com.vendor.burn.log.Log;
import com.vendor.burn.model.BoardConfig;
import com.vendor.burn.model.BurnContext;
import com.vendor.burn.model.FileSystem;
import com.vendor.burn.model.FlashType;
import com.vendor.burn.model.LocalConfig;
import com.vendor.burn.model.PartitionInfo;
import com.vendor.burn.model.PartitionItem;
import com.vendor.burn.model.TransferWay;
import com.vendor.burn.model.cmdline.BurnStatus;
import com.vendor.burn.model.cmdline.CmdLineParam;
import com.vendor.burn.model.cmdline.CmdType;
import com.vendor.burn.model.cmdline.SplitImgParam;
import com.vendor.burn.model.cmdline.adapter.BootBurn;
import com.vendor.burn.model.cmdline.adapter.EmmcImg;
import com.vendor.burn.model.cmdline.adapter.Erase;
import com.vendor.burn.model.cmdline.adapter.NandImg;
import com.vendor.burn.model.cmdline.adapter.SplitImg;
import com.vendor.burn.model.config.BurnConfig;
import com.vendor.burn.model.config.PreferenceConfig;
import com.vendor.burn.model.define.IBurner;
import com.vendor.burn.model.define.IFastbootProgressMonitor;
import com.vendor.burn.model.devices.burners.CommonBurner;
import com.vendor.burn.model.devices.burners.EmmcBurner;
import com.vendor.burn.model.devices.burners.FastbootBurner;
import com.vendor.burn.model.devices.unknown.UnknownDevice;
import com.vendor.burn.model.monitor.CmdMonitor;
import com.vendor.burn.packagers.nand.FlashOption;
import com.vendor.burn.packagers.nand.ip.NandIPCommon;
import com.vendor.burn.serial.workers.SerialHelper;
import com.vendor.burn.serial.workers.WorkerUtility;
import com.vendor.burn.sparse.SparseFileAgent;
import com.vendor.burn.sparse.SparseHeader;
import com.vendor.burn.utilitily.FileUtils;
import com.vendor.burn.utilitily.GZipUtils;
import com.vendor.burn.utilitily.PropertiesUtility;
import com.vendor.burn.utilitily.ToolUtils;
import com.vendor.burn.views.FastbootView;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.List;
import org.apache.commons.io.FileUtils;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.osgi.util.NLS;
import org.vendor.tftp.tool.TFTPService;
import org.xml.sax.SAXException;

public class CmdMain {
  private static final String VERSION = "5.6.27";
  
  private static final int EXTRA_START = 1;
  
  private static int EXTRA_COUNT = 1;
  
  private static CmdType curCmdType = null;
  
  private static String chipName = null;
  
  private static final int PARTITIONITEM_LENGHT_BY_ONE_IMAGE = 2;
  
  private static final int FILE_BLOCK_SIZE_32 = 32;
  
  private static final int FILE_UNIT_SIZE = 1048576;
  
  private static final int FILE_UNIT_SIZE_K = 1024;
  
  private static final int FILE_UNIT_SIZE_G = 1073741824;
  
  private IFastbootProgressMonitor progressMonitor;
  
  public static void main(String[] args) {
    if (args == null || args.length == 0) {
      help();
      System.exit(BurnStatus.PARAM_ERROR.getErrorCode());
    } 
    Log.println("BurnToolCLI Version: 5.6.27");
    ToolUtils.setCmdLine(true);
    CmdLineParam cmdLineParam = new CmdLineParam();
    com.vendor.burn.views.CmdMain cmdMain = new com.vendor.burn.views.CmdMain();
    if (!parseLogFileParentPath(args, cmdLineParam))
      System.exit(BurnStatus.PARAM_ERROR.getErrorCode()); 
    File dir = Log.getLogFileDir().toFile();
    if (!dir.exists() && 
      !dir.mkdirs()) {
      Log.println("mkdirs log file directory failed!");
      System.exit(0);
    } 
    File file = Log.getLogFile().toFile();
    if (file.exists() && 
      file.length() > 268435456L)
      FileUtils.deleteFile(file); 
    System.setProperty("logFileParentPath", Log.getLogFileDir().toString());
    if (!parseExtraParam(args, cmdLineParam))
      System.exit(BurnStatus.PARAM_ERROR.getErrorCode()); 
    int result = parseFirstParam(args, cmdLineParam, cmdMain);
    if (result != 0)
      System.exit(result); 
    System.exit(0);
  }
  
  public static CmdType getCurCmdType() {
    return curCmdType;
  }
  
  public static void setCurCmdType(CmdType curCmdType) {
    com.vendor.burn.views.CmdMain.curCmdType = curCmdType;
  }
  
  public static String getChipName() {
    return chipName;
  }
  
  public static void setChipName(String chipName) {
    com.vendor.burn.views.CmdMain.chipName = chipName;
  }
  
  private static boolean parseExtraParam(String[] args, CmdLineParam cmdLineParam) {
    if (args[0] == null)
      return true; 
    if (cmdLineParam == null) {
      Log.errorln(Messages.MESSAGE_EXTRA_PARAMETER_EMPTY);
      return false;
    } 
    int extraParam = args.length - 1;
    int index = 1;
    if (extraParam >= 1)
      while (index < extraParam) {
        String argsName = args[index];
        String argsValue = args[index + 1];
        if (argsName == null || argsValue == null) {
          Log.errorln(Messages.MESSAGE_UNEXPECTED_NULL);
          return false;
        } 
        if (!parseCmdParam(args, cmdLineParam, index, extraParam))
          return false; 
        if (!parseCommandParam(args, cmdLineParam, index))
          return false; 
        if (!parseParam(args, cmdLineParam, index, extraParam))
          return false; 
        index += EXTRA_COUNT;
      }  
    return true;
  }
  
  private static boolean parseParam(String[] args, CmdLineParam cmdLineParam, int index, int extraParam) {
    String argsName = args[index];
    String argsValue = args[index + 1];
    if (argsName.equalsIgnoreCase("-dt")) {
      if (argsValue.isEmpty()) {
        Log.errorln(NLS.bind(Messages.ERROR_COMMAND_PARAMETER, argsValue));
        return false;
      } 
      cmdLineParam.setBeforeBurningDelayTime(argsValue);
    } 
    if (argsName.equalsIgnoreCase("-s")) {
      if (argsValue.isEmpty()) {
        Log.errorln(NLS.bind(Messages.ERROR_COMMAND_PARAMETER, argsValue));
        return false;
      } 
      if (!parseCmdSocketParam(args, cmdLineParam, index, argsValue, extraParam))
        return false; 
    } 
    if (argsName.equalsIgnoreCase("-u")) {
      if (argsValue.isEmpty()) {
        Log.errorln(NLS.bind(Messages.ERROR_COMMAND_PARAMETER, argsValue));
        return false;
      } 
      if (!parseCmdUserNameParam(args, cmdLineParam, index, argsValue, extraParam))
        return false; 
    } 
    return true;
  }
  
  private static boolean parseLogFileParentPath(String[] args, CmdLineParam cmdLineParam) {
    for (int index = 0; index < args.length - 1; index++) {
      if (index == args.length - 1)
        return true; 
      String argsName = args[index];
      String argsValue = args[index + 1];
      if (argsName == null || argsValue == null) {
        Log.errorln(Messages.MESSAGE_UNEXPECTED_NULL);
        return false;
      } 
      if (argsName.equalsIgnoreCase("-lp")) {
        if (argsValue.isEmpty()) {
          Log.errorln(NLS.bind(Messages.ERROR_COMMAND_PARAMETER, argsValue));
          return false;
        } 
        CmdLineParam.setLogFileParentPath(argsValue);
      } 
    } 
    return true;
  }
  
  private static boolean parseCmdParam(String[] args, CmdLineParam cmdLineParam, int index, int extraParam) {
    String argsName = args[index];
    String argsValue = args[index + 1];
    if (argsName.equalsIgnoreCase("-n")) {
      if (argsValue.isEmpty()) {
        Log.errorln(NLS.bind(Messages.ERROR_COMMAND_PARAMETER, argsValue));
        return false;
      } 
      cmdLineParam.setChipName(argsValue);
    } 
    if (argsName.equalsIgnoreCase("-m")) {
      if (argsValue.isEmpty()) {
        Log.errorln(NLS.bind(Messages.ERROR_COMMAND_PARAMETER, argsValue));
        return false;
      } 
      cmdLineParam.setMode(argsValue);
      if (argsValue.equalsIgnoreCase("serial") || argsValue.equalsIgnoreCase("net") || argsValue
        .equalsIgnoreCase("NetNonNude")) {
        if (extraParam < index + 2) {
          Log.errorln(NLS.bind(Messages.ERROR_COMMAND_PARAMETER, "serialPort"));
          return false;
        } 
        String serialPort = args[index + 2];
        if (serialPort == null || serialPort.isEmpty()) {
          Log.errorln(NLS.bind(Messages.ERROR_COMMAND_PARAMETER, "serialPort"));
          return false;
        } 
        cmdLineParam.setSerialPort(serialPort);
        if (extraParam >= index + 3) {
          String baud = args[index + 3];
          cmdLineParam.setBaud(baud);
        } 
        if (!parseCmdNetParam(args, cmdLineParam, index, argsValue, extraParam))
          return false; 
      } 
    } 
    if (argsName.equalsIgnoreCase("-t")) {
      if (argsValue.isEmpty()) {
        Log.errorln(NLS.bind(Messages.ERROR_COMMAND_PARAMETER, argsValue));
        return false;
      } 
      cmdLineParam.setChipType(argsValue);
    } 
    return true;
  }
  
  private static boolean parseCmdSocketParam(String[] args, CmdLineParam cmdLineParam, int index, String argsValue, int extraParam) {
    if (extraParam < index + 2) {
      Log.errorln(NLS.bind(Messages.ERROR_COMMAND_PARAMETER, "serialServerIp orserialServerPort"));
      return false;
    } 
    String serialServerIp = args[index + 1];
    String serialServerPort = args[index + 2];
    if (!checkValid(serialServerIp))
      return false; 
    if (!checkValid(serialServerPort))
      return false; 
    cmdLineParam.setSocket(true);
    cmdLineParam.setSerialServerIp(serialServerIp);
    cmdLineParam.setSerialServerPort(serialServerPort);
    return true;
  }
  
  private static boolean parseCmdUserNameParam(String[] args, CmdLineParam cmdLineParam, int index, String argsValue, int extraParam) {
    if (extraParam < index + 2) {
      Log.errorln(NLS.bind(Messages.ERROR_COMMAND_PARAMETER, "username orpassword"));
      return false;
    } 
    String userName = args[index + 1];
    String passWord = args[index + 2];
    if (!checkValid(userName))
      return false; 
    if (passWord == null) {
      Log.errorln(NLS.bind(Messages.ERROR_COMMAND_PARAMETER, "password"));
      return false;
    } 
    cmdLineParam.setUserName(userName);
    cmdLineParam.setPassWord(passWord);
    return true;
  }
  
  private static boolean parseCommandParam(String[] args, CmdLineParam cmdLineParam, int index) {
    String argsName = args[index];
    String argsValue = args[index + 1];
    if (argsName.equalsIgnoreCase("-x")) {
      File file = new File(argsValue);
      if (!file.exists() || !file.isFile()) {
        Log.errorln(NLS.bind(Messages.ERROR_CURRENT_XML_PATH_NOT_FOUND, argsValue));
        return false;
      } 
      cmdLineParam.setXmlPath(argsValue);
      try {
        PartitionInfo PartitionInfo = PartitionItem.load(file, true);
        List<PartitionItem> partitionLists = PartitionInfo.getChildren();
        if (partitionLists != null && partitionLists.size() > 0) {
          cmdLineParam.setPartitionList(partitionLists);
        } else {
          Log.errorln(NLS.bind(Messages.ERROR_CURRENT_XML_PATH_NOT_FOUND, file.getCanonicalPath()));
          return false;
        } 
      } catch (DocumentException|IOException e) {
        Log.errorln(Messages.UanbleParseXml);
        return false;
      } 
    } else if (argsName.equalsIgnoreCase("-p")) {
      File file = new File(argsValue);
      if (!file.exists() || !file.isFile()) {
        Log.errorln(NLS.bind(Messages.ERROR_CURRENT_XML_PATH_NOT_FOUND, argsValue));
        return false;
      } 
      cmdLineParam.setProgrammerPath(argsValue);
    } else if (argsName.equalsIgnoreCase("-c")) {
      File file = new File(argsValue);
      if (!file.exists() || !file.isFile()) {
        Log.errorln(NLS.bind(Messages.ERROR_CURRENT_XML_PATH_NOT_FOUND, argsValue));
        return false;
      } 
      cmdLineParam.setBurnConfigPath(argsValue);
    } else if (argsName.equalsIgnoreCase("-b")) {
      File file = new File(argsValue);
      if (file.exists() && file.isFile()) {
        cmdLineParam.setBurnFilePath(argsValue);
      } else {
        Log.errorln(NLS.bind(Messages.ERROR_CURRENT_BURN_FILE_PATH_NOT_FOUND, argsValue));
        return false;
      } 
    } 
    return true;
  }
  
  private static boolean parseCmdNetParam(String[] args, CmdLineParam cmdLineParam, int index, String argsValue, int extraParam) {
    if (argsValue.equalsIgnoreCase("NetNonNude") || argsValue.equalsIgnoreCase("net")) {
      if (extraParam < index + 7) {
        Log.errorln(NLS.bind(Messages.ERROR_COMMAND_PARAMETER, "serverIp orboardIp or netMask or gatewayIp or tftpPort"));
        return false;
      } 
      String serverIp = args[index + 3];
      String boardIp = args[index + 4];
      String netMask = args[index + 5];
      String gatewayIp = args[index + 6];
      String tftpPort = args[index + 7];
      if (!checkValid(serverIp))
        return false; 
      if (!checkValid(boardIp))
        return false; 
      if (!checkValid(netMask))
        return false; 
      if (!checkValid(gatewayIp))
        return false; 
      if (!checkValid(tftpPort))
        return false; 
      cmdLineParam.setServerIp(serverIp);
      cmdLineParam.setBoardIp(boardIp);
      cmdLineParam.setNetMask(netMask);
      cmdLineParam.setGatewayIp(gatewayIp);
      cmdLineParam.setTftpPort(tftpPort);
    } 
    return true;
  }
  
  private static boolean checkValid(String argsValue) {
    if (argsValue == null || argsValue.isEmpty()) {
      Log.errorln(NLS.bind(Messages.ERROR_COMMAND_PARAMETER, argsValue));
      return false;
    } 
    return true;
  }
  
  private static int parseFirstParam(String[] args, CmdLineParam cmdLineParam, com.vendor.burn.views.CmdMain cmdMain) {
    String cmdType = args[0];
    if (cmdType == null)
      return BurnStatus.PARAM_ERROR.getErrorCode(); 
    if (cmdMain.checkCmdType(cmdType, "--help", "-h")) {
      setCurCmdType(CmdType.help);
      help();
    } else if (cmdMain.checkCmdType(cmdType, "--version", "-v")) {
      setCurCmdType(CmdType.version);
      printVersion();
    } else {
      if (cmdMain.checkCmdType(cmdType, "--burn", "-b"))
        return burn(cmdLineParam, cmdMain); 
      if (cmdMain.checkCmdType(cmdType, "--burnboot", "-bb"))
        return burnBoot(args, cmdMain, cmdLineParam); 
      if (cmdMain.checkCmdType(cmdType, "--erase", "-er"))
        return erase(args, cmdMain, cmdLineParam); 
      if (cmdMain.checkCmdType(cmdType, "--nandimg", "-n"))
        return makeNandImage(args, cmdMain, cmdLineParam); 
      if (cmdMain.checkCmdType(cmdType, "--emmcimg", "-e"))
        return makeEmmcImage(args, cmdMain, cmdLineParam); 
      if (cmdMain.checkCmdType(cmdType, "--splitimg", "-s"))
        return splitImage(args, cmdMain, cmdLineParam); 
      return BurnStatus.PARAM_ERROR.getErrorCode();
    } 
    return BurnStatus.SUCCESS.getErrorCode();
  }
  
  private boolean checkCmdType(String cmdType, String fullCmdName, String shortCmdName) {
    return (cmdType.equalsIgnoreCase(fullCmdName) || cmdType.equalsIgnoreCase(shortCmdName));
  }
  
  private static int makeNandImage(String[] args, com.vendor.burn.views.CmdMain cmdMain, CmdLineParam cmdLineParam) {
    setCurCmdType(CmdType.nandimg);
    NandImg nandImg = new NandImg(args);
    boolean parseResult = nandImg.parseAllParameter();
    if (!parseResult)
      return BurnStatus.PARAM_ERROR.getErrorCode(); 
    setChipName(nandImg.getChipName());
    NandIPCommon nandIp = nandImg.getNandIp();
    String inputFile = nandImg.getInputFile();
    boolean isFastboot = nandImg.isFastboot();
    FlashOption flashOption = nandImg.getFlashOption();
    FileSystem fileSystem = nandImg.getFileSystem();
    boolean isSelectRandom = nandImg.isRandomizer();
    int blockSize = nandImg.getBlock();
    PartitionItem item = new PartitionItem();
    item.setChecked(true);
    item.setFlashType(FlashType.nand);
    item.setFileSystem(fileSystem);
    item.setFile(new File(inputFile));
    if (isFastboot) {
      item.setStart(0L);
    } else {
      item.setStart(1L);
    } 
    PreferenceConfig preferenceConfig = new PreferenceConfig(cmdLineParam);
    if (!preferenceConfig.isLoadResult()) {
      Log.errorln(NLS.bind(Messages.MESSAGE_FAILED_To_LOAD_FILE, "preference.config"));
      return BurnStatus.LOAD_FILE_FAILED.getErrorCode();
    } 
    List<PartitionItem> partitions = new ArrayList<>();
    partitions.add(item);
    CmdMonitor cmdMonitor = new CmdMonitor();
    IFastbootProgressMonitor monitor = cmdMain.getProgressMonitor();
    String outputDir = (new File(inputFile)).getParent();
    try {
      nandIp.make(outputDir, flashOption, partitions, blockSize, isSelectRandom, monitor, (IProgressMonitor)cmdMonitor);
    } catch (ArrayIndexOutOfBoundsException e) {
      Log.errorln("Throws ArrayIndexOutOfBoundsException exception.");
      return BurnStatus.MAKE_NAND_IMAGE_FAILED.getErrorCode();
    } 
    return 0;
  }
  
  private static int burn(CmdLineParam cmdLineParam, com.vendor.burn.views.CmdMain cmdMain) {
    setCurCmdType(CmdType.burn);
    BurnContext burnContext = loadBurnConfig(cmdLineParam, null);
    if (burnContext == null) {
      Log.errorln(NLS.bind(Messages.MESSAGE_FAILED_To_LOAD_FILE, "burn.config"));
      return BurnStatus.LOAD_FILE_FAILED.getErrorCode();
    } 
    PreferenceConfig preferenceConfig = new PreferenceConfig(cmdLineParam);
    if (!preferenceConfig.isLoadResult()) {
      Log.errorln(NLS.bind(Messages.MESSAGE_FAILED_To_LOAD_FILE, "preference.config"));
      return BurnStatus.LOAD_FILE_FAILED.getErrorCode();
    } 
    setChipName(BurnConfig.getChipName());
    List<PartitionItem> partitionList = getPartitionList(cmdLineParam);
    if (partitionList == null)
      return BurnStatus.LOAD_FILE_FAILED.getErrorCode(); 
    String programmerPath = cmdLineParam.getProgrammerPath();
    boolean isBurnProgrammar = BurnConfig.isBurnProgrammer();
    File programmerFile = null;
    if (programmerPath != null && isBurnProgrammar) {
      programmerFile = new File(programmerPath);
      if (!programmerFile.exists())
        programmerFile = null; 
    } 
    if (BurnConfig.isCheckXml())
      for (PartitionItem part : partitionList) {
        if (part.getFile() != null && (!part.getFile().exists() || !part.getFile().isFile())) {
          Log.errorln(NLS.bind(Messages.MESSAGE_ERROR_FILE_PATH_DONT_EXIST, part.getName()));
          return BurnStatus.LOAD_FILE_FAILED.getErrorCode();
        } 
      }  
    boolean isBurnSplit = BurnConfig.isSplit();
    if (isBurnSplit)
      return cmdMain.burnZipByXML(partitionList, burnContext, programmerFile); 
    return cmdMain.burnByXML(partitionList, burnContext, programmerFile);
  }
  
  private int burnZipByXML(List<PartitionItem> partitionList, BurnContext burnContext, File programmerFile) {
    if (partitionList.size() > 0) {
      CommonBurner commonBurner = new CommonBurner(burnContext);
      ToolUtils.removeNoEarsePart(partitionList);
      commonBurner.setChildren(partitionList);
      if (programmerFile != null)
        try {
          commonBurner.setProgrammerFile(programmerFile.getCanonicalPath());
        } catch (IOException e) {
          return BurnStatus.IOEXCEPTION_ERROR.getErrorCode();
        }  
      if (checkStatus(commonBurner.checkBeforeBurn())) {
        IFastbootProgressMonitor monitor = getProgressMonitor();
        monitor.getEvent().setBurner((IBurner)commonBurner);
        return commonBurner.burnZip(monitor, commonBurner.getBurnerWorker());
      } 
    } else {
      Log.errorln(Messages.UanbleParseXml);
    } 
    return BurnStatus.LOAD_FILE_FAILED.getErrorCode();
  }
  
  private static List<PartitionItem> getPartitionList(CmdLineParam cmdLineParam) {
    List<PartitionItem> partitionList = null;
    String xmlPath = cmdLineParam.getXmlPath();
    if (xmlPath != null) {
      BurnConfig.setXmlPath(cmdLineParam.getXmlPath());
      partitionList = cmdLineParam.getPartitionList();
      if (partitionList == null) {
        Log.errorln(NLS.bind(Messages.MESSAGE_FAILED_To_LOAD_FILE, xmlPath));
        return null;
      } 
    } else {
      if (cmdLineParam.getBurnFilePath() != null)
        return getListByBurnFilePath(cmdLineParam); 
      String loadXmlPath = BurnConfig.getXmlPath();
      partitionList = loadPartitionList(cmdLineParam, loadXmlPath);
      if (partitionList == null)
        return null; 
    } 
    return partitionList;
  }
  
  private static List<PartitionItem> getListByBurnFilePath(CmdLineParam cmdLineParam) {
    List<PartitionItem> partitionList = new ArrayList<>();
    File file = new File(cmdLineParam.getBurnFilePath());
    try {
      FileUtils.checkFilePath(file);
    } catch (IOException e) {
      Log.errorln(NLS.bind(Messages.AddressBurner_msg_file_does_not_exist_or_is_a_directory, cmdLineParam
            .getBurnFilePath()));
      return null;
    } 
    if (file.length() == 0L) {
      Log.errorln(NLS.bind(Messages.AbstractBurner_msg_the_file_size_must_be_nonzero, cmdLineParam
            .getBurnFilePath()));
      return null;
    } 
    long complementsLenth = 1024L - file.length() % 1024L;
    for (int i = 0; i < 2; i++) {
      PartitionItem partitionItem = new PartitionItem();
      if (i == 0) {
        partitionItem.setName("fastboot");
        partitionItem.setStart(0L);
      } else {
        partitionItem.setStart(file.length() * i + complementsLenth);
      } 
      partitionItem.setChecked(true);
      partitionItem.setLength(file.length() + complementsLenth);
      partitionItem.setFile(file);
      partitionList.add(partitionItem);
    } 
    return partitionList;
  }
  
  private static int burnBoot(String[] args, com.vendor.burn.views.CmdMain cmdMain, CmdLineParam cmdLineParam) {
    setCurCmdType(CmdType.burnboot);
    int paramSize = args.length - 1;
    BootBurn bootBurn = new BootBurn();
    List<String> listSerialPorts = SerialHelper.getSerialPorts();
    bootBurn.setSerialPorts(listSerialPorts);
    boolean parseResult = bootBurn.parseAllParameter(args, paramSize, true);
    if (!parseResult)
      return BurnStatus.PARAM_ERROR.getErrorCode(); 
    setChipName(bootBurn.getChipName());
    BurnContext burnContext = loadAndCheckBurnConfig(bootBurn, cmdLineParam);
    PreferenceConfig preferenceConfig = new PreferenceConfig(cmdLineParam);
    if (!preferenceConfig.isLoadResult()) {
      Log.errorln(NLS.bind(Messages.MESSAGE_FAILED_To_LOAD_FILE, "preference.config"));
      return BurnStatus.LOAD_FILE_FAILED.getErrorCode();
    } 
    File bootFile = bootBurn.getBootFile();
    File programmer = bootBurn.getProgrammer();
    FastbootBurner fastbootBurner = new FastbootBurner(burnContext);
    try {
      if (programmer != null)
        fastbootBurner.setProgrammerFile(programmer.getCanonicalPath()); 
      if (bootFile != null)
        fastbootBurner.setFile(bootFile.getCanonicalPath()); 
    } catch (IOException e) {
      return BurnStatus.IOEXCEPTION_ERROR.getErrorCode();
    } 
    FlashType flashType = bootBurn.getFlashType();
    if (flashType != null)
      fastbootBurner.setFlashType(flashType); 
    if (checkStatus(fastbootBurner.checkBeforeBurn())) {
      IFastbootProgressMonitor monitor = cmdMain.getProgressMonitor();
      monitor.getEvent().setBurner((IBurner)fastbootBurner);
      return fastbootBurner.burnCmdLine(monitor);
    } 
    return BurnStatus.PARAM_ERROR.getErrorCode();
  }
  
  private static int erase(String[] args, com.vendor.burn.views.CmdMain cmdMain, CmdLineParam cmdLineParam) {
    setCurCmdType(CmdType.erase);
    int paramSize = args.length - 1;
    Erase earse = new Erase();
    List<String> listSerialPorts = SerialHelper.getSerialPorts();
    earse.setSerialPorts(listSerialPorts);
    boolean parseResult = earse.parseAllEarseParameter(args, paramSize, cmdLineParam);
    if (!parseResult)
      return BurnStatus.PARAM_ERROR.getErrorCode(); 
    setChipName(earse.getChipName());
    BurnContext burnContext = loadAndCheckEarseConfig(earse, cmdLineParam);
    PreferenceConfig preferenceConfig = new PreferenceConfig(cmdLineParam);
    if (!preferenceConfig.isLoadResult()) {
      Log.errorln(NLS.bind(Messages.MESSAGE_FAILED_To_LOAD_FILE, "preference.config"));
      return BurnStatus.LOAD_FILE_FAILED.getErrorCode();
    } 
    List<PartitionItem> partitionList = earse.getPartitionList();
    if (partitionList == null) {
      partitionList = getPartitionList(cmdLineParam);
      if (partitionList == null)
        return BurnStatus.LOAD_FILE_FAILED.getErrorCode(); 
    } 
    File programmerFile = earse.getProgrammer();
    return cmdMain.earseByXML(partitionList, burnContext, programmerFile);
  }
  
  private static BurnContext loadAndCheckBurnConfig(BootBurn bootBurn, CmdLineParam cmdLineParam) {
    BurnContext burnContext = loadBurnConfig(cmdLineParam, bootBurn.getChipName());
    if (burnContext == null) {
      Log.errorln(NLS.bind(Messages.MESSAGE_FAILED_To_LOAD_FILE, "burn.config"));
      System.exit(-1);
    } 
    LocalConfig localConfig = burnContext.getLocalConfig();
    String comPort = bootBurn.getPort();
    TransferWay mode = bootBurn.getMode();
    if (localConfig != null)
      localConfig.setComPort(comPort); 
    burnContext.setTransferWay(mode);
    burnContext.setJtagFrequency(BurnConfig.getBoardConfig().getFrequency());
    return burnContext;
  }
  
  private static BurnContext loadAndCheckEarseConfig(Erase earse, CmdLineParam cmdLineParam) {
    BurnContext burnContext = loadBurnConfig(cmdLineParam, earse.getChipName());
    if (burnContext == null) {
      Log.errorln(NLS.bind(Messages.MESSAGE_FAILED_To_LOAD_FILE, "burn.config"));
      System.exit(-1);
    } 
    LocalConfig localConfig = burnContext.getLocalConfig();
    String comPort = earse.getPort();
    TransferWay mode = earse.getMode();
    if (localConfig != null && comPort != null)
      localConfig.setComPort(comPort); 
    if (mode != null)
      burnContext.setTransferWay(mode); 
    if (getChipName() == null)
      setChipName(BurnConfig.getChipName()); 
    return burnContext;
  }
  
  private static int splitImage(String[] args, com.vendor.burn.views.CmdMain cmdMain, CmdLineParam cmdLineParam) {
    setCurCmdType(CmdType.splitimg);
    int paramSize = args.length - 1;
    SplitImg splitimg = new SplitImg();
    boolean parseResult = splitimg.parseAllParameter(args, paramSize);
    if (!parseResult)
      return BurnStatus.PARAM_ERROR.getErrorCode(); 
    List<PartitionItem> partitionList = splitimg.getPartitionList();
    String outputPath = splitimg.getOutputPath();
    File fileXml = new File(args[SplitImgParam.xmlPath.ordinal()]);
    File programe = new File(fileXml.getParent() + fileXml.getParent() + "programmer.bin");
    if (programe.exists()) {
      File newFile = new File(splitimg.getOutputPath() + splitimg.getOutputPath() + File.separator);
      try {
        FileUtils.copyFile(programe, newFile);
      } catch (IOException iOException) {}
    } 
    SAXReader reader = new SAXReader();
    Document oldDocument = null;
    try {
      try {
        reader.setFeature("http://apache.org/xml/features/disallow-doctype-decl", true);
      } catch (SAXException e) {
        Log.errorln("Failed to disable DTD!");
        return BurnStatus.LOAD_FILE_FAILED.getErrorCode();
      } 
      oldDocument = reader.read(fileXml);
    } catch (DocumentException e) {
      Log.errorln(NLS.bind(Messages.MESSAGE_ERROR_FILE_PATH_DONT_EXIST, fileXml.getName()));
    } 
    if (oldDocument == null) {
      Log.errorln(NLS.bind(Messages.MESSAGE_ERROR_FILE_PATH_DONT_EXIST, fileXml.getName()));
      return BurnStatus.LOAD_FILE_FAILED.getErrorCode();
    } 
    Element root = oldDocument.getRootElement();
    Document newDoc = DocumentHelper.createDocument();
    String programmerFile = root.attributeValue("ProgrammerFile");
    Element partitionInfo = newDoc.addElement("Partition_Info");
    partitionInfo.addAttribute("ProgrammerFile", programmerFile);
    List<Element> oldChildElements = root.elements();
    List<Element> newChildElements = partitionInfo.elements();
    ArrayList<FlashType> hasFlashTypes = FlashType.getCurFlashTypes(partitionList);
    boolean hasEmmcFlash = (isHasFlash(hasFlashTypes, FlashType.emmc) || isHasFlash(hasFlashTypes, FlashType.ufs) || isHasFlash(hasFlashTypes, FlashType.ssd));
    if (hasEmmcFlash) {
      long tempFileDivisionSize = getTempDivisionSize(splitimg.getDdrSize()) * 1048576L;
      if (tempFileDivisionSize == 0L) {
        Log.errorln(NLS.bind(Messages.CMD_PARAMETER_IsNull, "ddrSize"));
        return 0;
      } 
      for (int i = 0; i < partitionList.size(); i++) {
        PartitionItem item = partitionList.get(i);
        Element ele = oldChildElements.get(i);
        File file = item.getFile();
        if (file == null || !file.exists() || !item.isChecked()) {
          newChildElements.add((Element)ele.clone());
        } else {
          long filePartSize = file.length();
          if (filePartSize < 33554432L) {
            if (!file.getParent().equalsIgnoreCase(splitimg.getOutputPath())) {
              File newFile = new File(splitimg.getOutputPath() + splitimg.getOutputPath() + File.separator);
              try {
                FileUtils.copyFile(file, newFile);
              } catch (IOException iOException) {}
            } 
            newChildElements.add((Element)ele.clone());
          } else {
            FileSystem fileSystem = item.getFileSystem();
            boolean isSparse = (SparseFileAgent.isSparse(fileSystem) && SparseFileAgent.isAndroidSparseFile(file));
            if (filePartSize > tempFileDivisionSize) {
              boolean result = true;
              try {
                if (isSparse) {
                  result = createSparseTempFile(tempFileDivisionSize, outputPath, ele, newChildElements, item);
                } else {
                  result = createTempFile(i, file, tempFileDivisionSize, result, outputPath, ele, newChildElements, item
                      .getStart());
                } 
              } catch (IOException e) {
                Log.errorln(NLS.bind(Messages.ERROR_CURRENT_XML_PATH_NOT_FOUND, "split"));
                return BurnStatus.LOAD_FILE_FAILED.getErrorCode();
              } 
            } else {
              String partFilePath = outputPath + outputPath + File.separator + i;
              File partFile = new File(partFilePath);
              boolean copy = FileUtils.copy(null, file, partFile);
              if (!copy) {
                Log.errorln(NLS.bind(Messages.FASTBOOT_ERROR_Title, "copy file"));
                return BurnStatus.LOAD_FILE_FAILED.getErrorCode();
              } 
              Element newEle = (Element)ele.clone();
              if (isSparse) {
                RandomAccessFile inFile = null;
                try {
                  inFile = new RandomAccessFile(file, "r");
                  byte[] sparseHeaderData = new byte[28];
                  inFile.seek(0L);
                  int read = inFile.read(sparseHeaderData);
                  if (read < 0) {
                    Log.errorln(NLS.bind(Messages.FASTBOOT_ERROR_Title, "Cannot read sparse header, file format error."));
                    return BurnStatus.LOAD_FILE_FAILED.getErrorCode();
                  } 
                  SparseHeader sparseHeader = new SparseHeader();
                  sparseHeader.fromBytes(sparseHeaderData);
                  long tempFileBurnNeedSize = sparseHeader.blk_sz * sparseHeader.total_blks;
                  File zipTempFile = GZipUtils.compress(partFile, true, tempFileBurnNeedSize, true, 0L, item
                      .getStart());
                  if (zipTempFile == null)
                    return BurnStatus.LOAD_FILE_FAILED.getErrorCode(); 
                  newEle.addAttribute("SelectFile", zipTempFile.getName());
                  newChildElements.add(newEle);
                  if (partFile != null && partFile.exists())
                    FileUtils.deleteFile(partFile); 
                } catch (Exception exception) {
                
                } finally {
                  if (inFile != null)
                    try {
                      inFile.close();
                    } catch (IOException iOException) {} 
                } 
              } else {
                File zipTempFile = GZipUtils.compress(partFile, true, file.length(), false, 0L, item.getStart());
                if (zipTempFile == null)
                  return BurnStatus.LOAD_FILE_FAILED.getErrorCode(); 
                newEle.addAttribute("SelectFile", zipTempFile.getName());
                newChildElements.add(newEle);
              } 
              if (partFile != null && partFile.exists()) {
                boolean isDelete = FileUtils.deleteFile(partFile);
                if (!isDelete) {
                  Log.errorln(NLS.bind(Messages.FASTBOOT_ERROR_Title, "delete file"));
                  return BurnStatus.LOAD_FILE_FAILED.getErrorCode();
                } 
              } 
            } 
          } 
        } 
      } 
      writeXml(newDoc, outputPath, fileXml);
    } 
    return BurnStatus.SUCCESS.getErrorCode();
  }
  
  private static long getTempDivisionSize(long ddrFreeSize) {
    if (ddrFreeSize / 2L > 32L)
      return ddrFreeSize / 2L; 
    return 0L;
  }
  
  private static boolean writeXml(Document doc, String path, File oldFile) {
    OutputFormat format = OutputFormat.createPrettyPrint();
    format.setEncoding("UTF-8");
    int index = oldFile.getName().indexOf(".xml");
    String name = oldFile.getName().substring(0, index);
    File newFile = new File(path + path + File.separator + "_split.xml");
    XMLWriter writer = null;
    try {
      writer = new XMLWriter(FileUtils.openOutputStream(newFile), format);
      writer.write(doc);
    } catch (IOException e) {
      Log.errorln(NLS.bind(Messages.ERROR_GENERATE_XML, e));
      return false;
    } finally {
      if (writer != null)
        try {
          writer.close();
        } catch (IOException iOException) {} 
    } 
    return true;
  }
  
  private static boolean createTempFile(int partIndex, File file, long tempFileDivisionSize, boolean result, String path, Element ele, List<Element> newChildElements, long partStart) throws FileNotFoundException, IOException {
    FileInputStream inputStream = null;
    int tempTimes = 0;
    try {
      if (file != null && file.exists() && file.isFile()) {
        inputStream = new FileInputStream(file);
        byte[] buffer = new byte[2097152];
        long totalLength = file.length();
        long startAddrOffset = 0L;
        while (result && totalLength > 0L) {
          File partfile = null;
          long partLength = 0L;
          try {
            int read = -1;
            String partFilePath = path + path + File.separator + partIndex + ".part" + file.getName();
            partfile = new File(partFilePath);
            boolean createNewFile = partfile.createNewFile();
            if (!createNewFile)
              System.out.println("Failed to create file " + partfile
                  .getName()); 
            OutputStream outputtream = null;
            try {
              outputtream = FileUtils.openOutputStream(partfile);
              int length = (int)Math.min(tempFileDivisionSize, buffer.length);
              while (partLength < tempFileDivisionSize && result && (
                
                read = inputStream.read(buffer, 0, length)) > 0) {
                outputtream.write(buffer, 0, read);
                partLength += read;
                if (tempFileDivisionSize - partLength < buffer.length) {
                  length = (int)(tempFileDivisionSize - partLength);
                  continue;
                } 
                length = buffer.length;
              } 
            } finally {
              if (outputtream != null)
                outputtream.close(); 
            } 
            File zipTempFile = GZipUtils.compress(partfile, true, partLength, false, startAddrOffset, partStart);
            if (zipTempFile == null)
              return false; 
            Element newEle = (Element)ele.clone();
            long start = 0L;
            newEle.addAttribute("Length", convertUnit(partLength));
            if (tempTimes > 1) {
              Element preEle = newChildElements.get(newChildElements.size() - 1);
              long len = WorkerUtility.ReceiveHelper.convertToNumber(preEle.attributeValue("Length"));
              long preStart = WorkerUtility.ReceiveHelper.convertToNumber(preEle.attributeValue("Start"));
              start = len + preStart;
              newEle.addAttribute("Start", convertUnit(start));
            } 
            newEle.addAttribute("SelectFile", zipTempFile.getName());
            newChildElements.add(newEle);
            startAddrOffset += partLength;
            totalLength -= partLength;
          } finally {
            if (partfile != null && partfile.exists())
              FileUtils.deleteFile(partfile); 
          } 
        } 
      } 
    } finally {
      if (inputStream != null)
        inputStream.close(); 
    } 
    return result;
  }
  
  private static boolean createSparseTempFile(long ddrUsableSize, String outputPath, Element ele, List<Element> newChildElements, PartitionItem partitionItem) throws FileNotFoundException, IOException {
    List<PartitionItem> partitionItemList = SparseFileAgent.getSparseFileXmlList(ddrUsableSize, partitionItem, false);
    if (partitionItemList.size() == 0)
      return false; 
    File tmpFile = null;
    long offset = 0L;
    long partStart = partitionItem.getStart();
    try {
      for (int i = 0; i < partitionItemList.size(); i++) {
        PartitionItem item = partitionItemList.get(i);
        long fileBurnNeedSize = item.getLength();
        tmpFile = item.getFile();
        File zipTempFile = GZipUtils.compress(tmpFile, true, fileBurnNeedSize, true, offset, partStart);
        if (zipTempFile == null)
          return false; 
        Element newEle = (Element)ele.clone();
        newEle.addAttribute("SelectFile", zipTempFile.getName());
        newEle.addAttribute("Length", convertUnit(fileBurnNeedSize));
        if (i >= 1) {
          Element preEle = newChildElements.get(newChildElements.size() - 1);
          long len = WorkerUtility.ReceiveHelper.convertToNumber(preEle.attributeValue("Length"));
          long preStart = WorkerUtility.ReceiveHelper.convertToNumber(preEle.attributeValue("Start"));
          long start = len + preStart;
          newEle.addAttribute("Start", convertUnit(start));
        } 
        newChildElements.add(newEle);
        offset += fileBurnNeedSize;
        if (!FileUtils.deleteFile(tmpFile))
          Log.warningln(NLS.bind(Messages.FileDeleteFailed, tmpFile)); 
      } 
    } finally {
      if (!FileUtils.deleteFile(tmpFile))
        Log.warningln(NLS.bind(Messages.FileDeleteFailed, tmpFile)); 
    } 
    return true;
  }
  
  private static boolean isHasFlash(ArrayList<FlashType> hasFlashTypes, FlashType curFlashType) {
    for (FlashType flashType : hasFlashTypes) {
      if (FlashType.fuzzyCompareFlash(flashType, curFlashType))
        return true; 
    } 
    return false;
  }
  
  private static int makeEmmcImage(String[] args, com.vendor.burn.views.CmdMain cmdMain, CmdLineParam cmdLineParam) {
    setCurCmdType(CmdType.emmcimg);
    int paramSize = args.length - 1;
    EmmcImg emmcImg = new EmmcImg();
    boolean parseResult = emmcImg.parseAllParameter(args, paramSize);
    if (!parseResult)
      return BurnStatus.PARAM_ERROR.getErrorCode(); 
    setChipName(getChipName());
    PreferenceConfig preferenceConfig = new PreferenceConfig(cmdLineParam);
    if (!preferenceConfig.isLoadResult()) {
      Log.errorln(NLS.bind(Messages.MESSAGE_FAILED_To_LOAD_FILE, "preference.config"));
      System.exit(-1);
    } 
    List<PartitionItem> partitionList = emmcImg.getPartitionList();
    UnknownDevice device = emmcImg.getDevice();
    String outputPath = emmcImg.getOutputPath();
    if (device != null) {
      BurnContext burnContext = device.getBurnContext();
      EmmcBurner emmcBurner = new EmmcBurner(burnContext);
      emmcBurner.setChildren(partitionList);
      if (emmcBurner.checkCreateImage() == null) {
        IFastbootProgressMonitor monitor = cmdMain.getProgressMonitor();
        monitor.getEvent().setBurner((IBurner)emmcBurner);
        emmcBurner.setEmmcImgOutPath(outputPath);
        return emmcBurner.createImageCmdLine(monitor);
      } 
      return BurnStatus.NULL_POINTER_ERROR.getErrorCode();
    } 
    return 0;
  }
  
  private static List<PartitionItem> loadPartitionList(CmdLineParam cmdLineParam, String xmlPath) {
    List<PartitionItem> partitionLists = null;
    try {
      File file = new File(xmlPath);
      if (!file.exists() || !file.isFile()) {
        Log.errorln(NLS.bind(Messages.ERROR_CURRENT_XML_PATH_NOT_FOUND, xmlPath));
        return null;
      } 
      String xmlPathDefault = PropertiesUtility.queryForString("xmlPathDefault");
      if (xmlPathDefault != null && !xmlPathDefault.isEmpty()) {
        PartitionInfo PartitionInfo = PartitionItem.load(file, Boolean.valueOf(xmlPathDefault.trim()).booleanValue());
        partitionLists = PartitionInfo.getChildren();
        File programmerFile = PartitionInfo.getProgrammerFile();
        if (programmerFile != null)
          try {
            cmdLineParam.setProgrammerPath(PartitionInfo.getProgrammerFile().getCanonicalPath());
          } catch (IOException e) {
            Log.errorln(Messages.UanbleParseXml);
          }  
      } else {
        Log.errorln(NLS.bind(Messages.MESSAGE_FAILED_To_READ_KEY, "xmlPathDefault"));
        return null;
      } 
      if (partitionLists != null && partitionLists.size() > 0) {
        cmdLineParam.setPartitionList(partitionLists);
      } else {
        Log.errorln(NLS.bind(Messages.MESSAGE_FAILED_To_LOAD_FILE, file.getName()));
        return null;
      } 
    } catch (DocumentException e) {
      Log.errorln(Messages.UanbleParseXml);
      return null;
    } 
    return partitionLists;
  }
  
  private int burnByXML(List<PartitionItem> partitionList, BurnContext burnContext, File programmerFile) {
    if (partitionList.size() > 0) {
      CommonBurner commonBurner = new CommonBurner(burnContext);
      List<PartitionItem> parts = ToolUtils.removeNoEarsePart(partitionList);
      commonBurner.setChildren(parts);
      if (programmerFile != null)
        try {
          commonBurner.setProgrammerFile(programmerFile.getCanonicalPath());
        } catch (IOException e) {
          return BurnStatus.IOEXCEPTION_ERROR.getErrorCode();
        }  
      if (checkStatus(commonBurner.checkBeforeBurn())) {
        IFastbootProgressMonitor monitor = getProgressMonitor();
        monitor.getEvent().setBurner((IBurner)commonBurner);
        return commonBurner.commonBurn(monitor, commonBurner.getBurnerWorker());
      } 
    } else {
      Log.errorln(Messages.UanbleParseXml);
    } 
    return BurnStatus.LOAD_FILE_FAILED.getErrorCode();
  }
  
  private int earseByXML(List<PartitionItem> partitionList, BurnContext burnContext, File programmerFile) {
    if (partitionList.size() > 0) {
      CommonBurner commonBurner = new CommonBurner(burnContext);
      commonBurner.setChildren(partitionList);
      if (programmerFile != null)
        try {
          commonBurner.setProgrammerFile(programmerFile.getCanonicalPath());
        } catch (IOException e) {
          return BurnStatus.IOEXCEPTION_ERROR.getErrorCode();
        }  
      if (checkStatus(commonBurner.checkBeforeBurn())) {
        IFastbootProgressMonitor monitor = getProgressMonitor();
        monitor.getEvent().setBurner((IBurner)commonBurner);
        return commonBurner.erase(monitor, commonBurner.getBurnerWorker());
      } 
    } else {
      Log.errorln(Messages.UanbleParseXml);
    } 
    return BurnStatus.LOAD_FILE_FAILED.getErrorCode();
  }
  
  private static boolean checkStatus(IStatus status) {
    if (status == Status.CANCEL_STATUS)
      return false; 
    boolean valid = (status == null || status.isOK());
    if (!valid)
      Log.errorln(status.getMessage()); 
    return valid;
  }
  
  private static BurnContext loadBurnConfig(CmdLineParam cmdLineParam, String chipName) {
    BurnConfig burnConfig = new BurnConfig(cmdLineParam);
    if (!burnConfig.isLoadResult())
      return null; 
    String deviceName = BurnConfig.getChipName();
    if (chipName != null)
      deviceName = chipName; 
    setChipName(BurnConfig.getChipName());
    Log.println("The current select chip: " + deviceName);
    UnknownDevice ud = null;
    if (FastbootView.isChipBranchSeries5()) {
      ud = new UnknownDevice();
    } else {
      ud = new UnknownDevice(deviceName);
      if (!ud.isValid()) {
        Log.errorln(Messages.FastbootView_Unsupported_Device);
        return null;
      } 
    } 
    LocalConfig localConfig = ud.getBurnContext().getLocalConfig();
    BoardConfig boardConfig = ud.getBurnContext().getBoardConfig();
    localConfig.setComPort(BurnConfig.getLocalConfig().getComPort());
    localConfig.setServerIp(BurnConfig.getLocalConfig().getServerIp());
    localConfig.setRemoteTftpServer(BurnConfig.getLocalConfig().isRemoteTftpServer());
    localConfig.setRemoteTftpServerDir(BurnConfig.getLocalConfig().getRemoteTftpServerDir());
    boardConfig.setSocket(BurnConfig.getBoardConfig().isSocket());
    boardConfig.setSerialServerIp(BurnConfig.getBoardConfig().getSerialServerIp());
    boardConfig.setSerialServerPort(BurnConfig.getBoardConfig().getSerialServerPort());
    boardConfig.setIp(BurnConfig.getBoardConfig().getIp());
    boardConfig.setGateway(BurnConfig.getBoardConfig().getGateway());
    boardConfig.setSubnetMask(BurnConfig.getBoardConfig().getSubnetMask());
    boardConfig.setMac(BurnConfig.getBoardConfig().getMac());
    boardConfig.setWriteEthaddr(BurnConfig.getBoardConfig().isWriteEthaddr());
    boardConfig.setUdpPort(BurnConfig.getBoardConfig().getUdpPort());
    boardConfig.setUserName(BurnConfig.getBoardConfig().getUserName());
    boardConfig.setPassWord(BurnConfig.getBoardConfig().getPassWord());
    ud.getBurnContext().setTransferWay(BurnConfig.getTransferway());
    ud.getBurnContext().setDwTimeout(BurnConfig.getDwTimeout());
    ud.getBurnContext().setJtagFrequency(BurnConfig.getBoardConfig().getFrequency());
    ud.getBurnContext().setUsbToEthernet(BurnConfig.isUSBToEthernet());
    if (TransferWay.isNet(BurnConfig.getTransferway()))
      TFTPService.setUsb2EtherNet((BurnConfig.getTransferway() == TransferWay.UsbTransNet || 
          BurnConfig.isUSBToEthernet())); 
    return ud.getBurnContext();
  }
  
  private static void help() {
    printBurnByXml();
    Log.println("");
    BootBurn.printBurnBootEg();
    Log.println("");
    Erase.printEarseBootEg();
    Log.println("");
    NandImg.printNandImgEg();
    Log.println(Messages.NAND_IMG_GET_PAGESIZE_AND_BLOCK);
    EmmcImg.printEmmcImgEg();
    Log.println("");
    Log.println("");
    Log.println("general options:");
    Log.println("--help    | -h    usage message");
    Log.println("--version | -v    version number and copyright");
    Log.println("--burn    | -b    this command will burn image to the flash according to the partition table xmlPath.");
    Log.println("                  first of all, you need to configure the burn.config file.");
    Log.println("--burnboot| -bb   This command is used to burn fastboot.");
    Log.println("--nandimg | -n    the commands is used to create nand image.");
    Log.println("--emmcimg | -e    the commands is used to create emmc image.");
  }
  
  private static void printVersion() {
    Log.println(Messages.MESSAGE_BURN_VERSION_NAME + "5.6.27");
  }
  
  private static void printBurnByXml() {
    Log.println("burntoolcli.jar --burn [-x xmlPath] [-p programmerPath]");
    Log.println(Messages.MESSAGE_BURN_BURN_BY_XML);
    Log.println(Messages.MESSAGE_BURN_BURN_NOTICE);
  }
  
  private IFastbootProgressMonitor getProgressMonitor() {
    if (this.progressMonitor == null)
      this.progressMonitor = IFastbootProgressMonitor.PROGRESSMONITORCMDLINE; 
    return this.progressMonitor;
  }
  
  private static String convertUnit(long bits) {
    String result = null;
    if (bits % 1073741824L == 0L) {
      result = "" + bits / 1073741824L + "G";
    } else if (bits % 1048576L == 0L) {
      result = "" + bits / 1048576L + "M";
    } else if (bits % 1024L == 0L) {
      result = "" + bits / 1024L + "K";
    } else {
      result = "" + bits + "b";
    } 
    return result;
  }
}
